/**
* Classe representant une addition
*
* @version 0.1, 06/05/2003
* @see Expr
* @see java.lang.String
* @see TEntier
* @see VEntier
* @see Env
*/
public class Addition extends Expr
{
	private Expr e1;
	private Expr e2;

	/**
	* Constructeur de la classe Addition
	*
	* @param Les deux expressions a droite et a gauche de l'Addition.
	*/
	public Addition (Expr e1, Expr e2)
	{
		this.e1 = e1;
		this.e2 = e2;
	}

	/**
	* Renvoit la String representant l'addition.
	*
	* @return Une String representant l'addition.
	*/
	public String expr_str ()
	{
		return ("(" + e1.expr_str () + " + " + e2.expr_str () + ")");
	}

	/**
	* Calcule le type Type issu de l'Addition.
	*
	* @param L'environnement. 
	* @return Le Type de cette addition.
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Type typer (Env envp) throws Exception
	{
		if (TEntier.isTEntier((e1.typer (envp))) == false) {
			throw new TypeException("Addition: Erreur de typage: " + e1.expr_str () + " n'est pas entiere.");
		}

		if (TEntier.isTEntier((e2.typer(envp))) == false) {
			throw new TypeException("Addition: Erreur de typage: " + e2.expr_str () + " n'est pas entiere.");
		}

		return (new TEntier ());
	}

	/**
	* Calcule la Valeur issue de l'Addition.
	*
	* @param L'environnement
	* @return La Valeur de cette Addition.
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Valeur evaluer (Env envp) throws Exception
	{
		if (TEntier.isTEntier((e1.typer (envp))) == false) {
			throw new TypeException("Addition: Erreur de typage: " + e1.expr_str () + " n'est pas entiere.");
		}

		if (TEntier.isTEntier((e2.typer(envp))) == false) {
			throw new TypeException("Addition: Erreur de typage: " + e2.expr_str () + " n'est pas entiere.");
		}

		return (new VEntier(e1.evaluer(envp).valeur_int() + e2.evaluer(envp).valeur_int()));
	}
}
