/**
* Classe application: Application d'une fonction a une variable.
*
* @version 0.2, 07/05/2003
* @see Expr
*/
public class Application extends Expr
{
	private Expr e1;
	private Expr e2;

	/**
	* Constructeur d'Application: prend les 2 expressions, de gauche et de 
	* droite
	*	
	* @param Les deux expressions.
	*/
	public Application (Expr e1, Expr e2)
	{
		this.e1 = e1;
		this.e2 = e2;
	}

	/**
	* Renvoit la String representant l'expression de l'application.
	*
	* @return La String.
	*/
	public String expr_str ()
	{
		return ("(" + e1.expr_str() + " " + e2.expr_str() + ")");
	}

	/**
	* Renvoit le Type resultant de l'Application.
	*
	* @param L'environnement.
	* @return Le Type.
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Type typer (Env envp) throws Exception
	{
		Type nt = e1.typer(envp);
		Type nt2 = e2.typer(envp);

		if (nt instanceof TFonction){
			if (TFonction.compareType(((TFonction)nt).get_t_gauche(), nt2) == true) {
				return (((TFonction)nt).get_t_droite());
			} else {
				throw new TypeException ("Application: Erreur de typage: " + e2.expr_str () + " n'est pas le type attendu.");
			}
		} else {
			throw new TypeException ("Application: Erreur de typage: " + e1.expr_str () + "n'est pas une fonction.");
		}
	}

	/**
	* Renvoit la Valeur resultant de l'Application.
	*
	* @param envp L'environnement.
	* @return La Valeur
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Valeur evaluer (Env envp) throws Exception
	{
		Valeur tmp_val;
		Valeur val = e1.evaluer (envp);
		Env tmp_env;

		if (val instanceof VFermeture) {

			tmp_env = envp.MergeWithEnv(((VFermeture)val).getEnv());

			/* On evalue e2 dans envp */
			tmp_val = e2.evaluer (envp);

			tmp_env.ajt_env (((VFermeture)val).getString(), ((VFermeture)val).getType(), tmp_val);
			tmp_val = ((VFermeture)val).getExpr().evaluer(tmp_env);

			return (tmp_val);
		} else {
			throw new ValeurException ("evaluer");
		}
	}
}
