/**
* Classe representant une difference (comparaison)
*
* @version 0.1, 06/05/2003
* @see Expr
* @see java.lang.String
* @see TEntier
* @see VEntier
* @see TBool
* @see VBool
* @see Env
*/
public class Different extends Expr
{
	private Expr e1;
	private Expr e2;

	/**
	* Constructeur pour Different
	*
	* @param Les deux expressions a droite et a gauche de la difference
	*/
	public Different (Expr e1, Expr e2)
	{
		this.e1 = e1;
		this.e2 = e2;
	}

	/**
	* Renvoit la String representant la difference.
	*
	* @return: La String.
	*/
	public String expr_str ()
	{
		return ("(" + e1.expr_str () + " <> " + e2.expr_str () + ")");
	}

	/**
	* Renvoit le Type resultant de la difference.
	*
	* @param l'environnement.
	* @return le Type de l'expression
	* @exception TypeException
	*            Erreur de typage
	*/
	public Type typer (Env envp) throws Exception
	{
		if (TBool.isTBool((e1.typer (envp)))) { /* e1 bool */
			if (TBool.isTBool((e2.typer(envp))) == false) { /* e2 pas bool */
				throw new TypeException("Different: Erreur de typage: " + e2.expr_str () + " n'est pas booleene.");
			}
			return (new TBool ());
		}

		if (TEntier.isTEntier((e1.typer (envp)))) { /* e1 int */
			if (TEntier.isTEntier((e2.typer(envp))) == false) { /* pas int */
				throw new TypeException("Different: Erreur de typage: " + e2.expr_str () + " n'est pas entier.");
			}
			return (new TBool ());
		}

		throw new TypeException("Different: Erreur de typage: " + e1.expr_str() + " n'est pas reconnue.");

	}

	/**
	* Renvoit la Valeur resultante de la difference.
	*
	* @param l'environnement
	* @return le Type de l'expression
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Valeur evaluer (Env envp) throws Exception
	{
		if (TBool.isTBool((e1.typer (envp)))) {
			if (TBool.isTBool((e2.typer(envp))) == false) {
				throw new TypeException("Different: Erreur de typage: " + e1.expr_str () + " n'est pas booleene.");
			}
			return (new VBool(e1.evaluer(envp).valeur_bool() != e2.evaluer(envp).valeur_bool()));
		}

		if (TEntier.isTEntier((e1.typer (envp)))) { /* e1 int */
			if (TEntier.isTEntier((e2.typer(envp))) == false) { /* pas int */
				throw new TypeException("Different: Erreur de typage: " + e2.expr_str () + " n'est pas entier.");
			}
			return (new VBool(e1.evaluer(envp).valeur_int() != e2.evaluer(envp).valeur_int()));
		}

		throw new TypeException("Different: Erreur de typage: " + e1.expr_str() + " n'est pas reconnue.");
	}
}
