/**
* Classe representant une egalite 
*
* @version 0.1, 06/05/2003
* @see Expr
* @see java.lang.String
* @see TEntier
* @see VEntier
* @see TBool
* @see VBool
* @see Env
*
*/
public class Egal extends Expr
{
	private Expr e1;
	private Expr e2;

	/**
	* Constructeur de la classe Egal
	*
	* @param Les deux expressions a droite et a gauche de l'egalite.
	*/
	public Egal (Expr e1, Expr e2)
	{
		this.e1 = e1;
		this.e2 = e2;
	}

	/**
	* Renvoit la String representant l'egalite.
	*
	* @return Une String representant l'egalite.
	*/
	public String expr_str ()
	{
		return ("(" + e1.expr_str () + " = " + e2.expr_str () + ")");
	}

	/**
	* Calcule le type Type issu de l'egalite
	*
	* @param L'environnement. 
	* @return Le Type de cette egalite.
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Type typer (Env envp) throws Exception
	{
		if (TBool.isTBool((e1.typer (envp)))) { /* e1 bool */
			if (TBool.isTBool((e2.typer(envp))) == false) { /* e2 pas bool */
				throw new TypeException("Egal: Erreur de typage: " + e2.expr_str () + " n'est pas booleene.");
			}
			return (new TBool ());
		}

		if (TEntier.isTEntier((e1.typer (envp)))) { /* e1 int */
			if (TEntier.isTEntier((e2.typer(envp))) == false) { /* pas int */
				throw new TypeException("Egal: Erreur de typage: " + e2.expr_str () + " n'est pas entier.");
			}
			return (new TBool ());
		}

		throw new TypeException("Egal: Erreur de typage: " + e1.expr_str() + " n'est pas reconnue.");

	}


	/**
	* Calcule la Valeur issue de l'egalite.
	*
	* @param L'environnement
	* @return La Valeur de cette Egalite (VBool).
	* @exception TypeException
	*            Erreur de typage
	*/
	public Valeur evaluer (Env envp) throws Exception
	{
		if (TBool.isTBool((e1.typer (envp)))) {
			if (TBool.isTBool((e2.typer(envp))) == false) {
				throw new TypeException("Egal: Erreur de typage: " + e1.expr_str () + " n'est pas booleene.");
			}
			return (new VBool(e1.evaluer(envp).valeur_bool() == e2.evaluer(envp).valeur_bool()));
		}

		if (TEntier.isTEntier((e1.typer (envp)))) { /* e1 int */
			if (TEntier.isTEntier((e2.typer(envp))) == false) { /* pas int */
				throw new TypeException("Egal: Erreur de typage: " + e2.expr_str () + " n'est pas entier.");
			}
			return (new VBool(e1.evaluer(envp).valeur_int() == e2.evaluer(envp).valeur_int()));
		}

		throw new TypeException("Egal: Erreur de typage: " + e1.expr_str() + " n'est pas reconnue.");
	}
}
