/**
* Le type de classe abstraite representant une Expression.
*
* @version 0.2, 07/05/2003
* @see java.io.Serializable
* @see java.lang.Object
* @see java.lang.String
* @see Type
* @see Valeur
* @see Exception
*/
abstract class Expr implements java.io.Serializable
{
	/**
	* Retourne une chaine de caracteres representant l'expression.
	*
	* @return La chaine de caracteres.
	*/
	abstract String expr_str ();

	/* Methode non abstraite, mais dans le LetIn on la redfini. */
	public String expr_str(Env envp) throws Exception
	{
		return (this.expr_str());
	}

	/**
	* Calcule et retourne l'objet de type Type typant l'expression. 
	* Si une erreur intervient, une exception peut etre elevee.
	*
	* @param Env envp, l'envirronement dans lequel on effectue les calculs.
	* @return Un objet de type Type.
	*/
	abstract Type typer (Env envp) throws Exception;

	/**
	* Calcule et retourne l'objet de type Valeur valorisant l'expression.
	* Si une erreur intervient, une exception peut etre elevee.
	*
	* @param Env envp, l'envirronement dans lequel on effectue les calculs.
	* @return Un objet de type Valeur.
	*/
	abstract Valeur evaluer (Env envp) throws Exception;

	/**
	* Affiche la chaine de caracteres reprensentant l'expression.
	*/
	public void afficher_expr ()
	{
		System.out.println (this);
	}

	/**
	* Retourne une chaine de caracteres reprensentant l'expression.
	*
	* @return La chaine de caracteres.
	*/
	public String toString ()
	{
		return (this.expr_str ());
	}

	/**
	* Execution d'une expression
	*
	* @param env L'environnement.
	*/
	public void executer (Env env)
	{
		StringBuffer sb = new StringBuffer ("");
		try {
			sb.append("# " + this.expr_str(env));
			sb.append(" ;;");
			sb.append("\n- : " + this.typer(env).type_str());
			sb.append(" = ");
			sb.append(this.evaluer(env).val_str());
			System.out.println (sb.toString ());
		} catch (Exception ee) {
			System.out.println (sb.toString ());
			System.out.println (ee);
		}
		return;
	}
}
