/**
* Classe representant une fonction
*
* @version 0.1, 06/05/2003
* @see Expr
* @see java.lang.String
* @see Env
*/
public class Fonction extends Expr
{
	String s;
	Type t;
	Expr e;

	/**
	* Constructeur de Fonction
	*
	* param String s, nom de la variable de la fonction
	* param Type t, type de la variable de la fonction
	* param Expr e, expression de la fonction.
	*/
	public Fonction (String s, Type t, Expr e)
	{
		this.s = s;
		this.t = t;
		this.e = e;
	}

	/**
	* Renvoit la String reprensentant la Fonction
	*
	* @return Une String representant la Fonction
	*/
	public String expr_str ()
	{
		return ("(fun " + s + " -> " + e.expr_str() + ")");
	}

	/**
	* Renvoit le Type de cette Fonction (Tfonction)
	*
	* @param L'environnement
	* @return Une instance complete de TFonction.
	*/
	public Type typer (Env envp) throws Exception
	{
		Env tmp_env;
		Type tr;

		/* Dans la seconde partie, on a l'environement envp, auquel on 
		   rajoute "s,t" */
		/* Suite a la modification de Env, on doit plus utiliser l'env
		   courant; On procede a une copine qui sera detruite apres. */

		tmp_env = Env.CopieEnv(envp);
		tmp_env.ajt_env_typ (s, t);
		tr = new TFonction (t, e.typer(tmp_env));

		return (tr);
	}

	/**
	* Renvoit la Valeur de cette fonction (VFermeture)
	*
	* @param L'environnement
	* @return Une instance complete de VFermeture.
	*/
	public Valeur evaluer (Env envp) throws Exception
	{
		Env tmp_env;
		// s, t et e ne changent pas. Ce qui n'est pas le cas de envp 

		tmp_env = Env.CopieEnv (envp);
		// Il y a deja un 's' dans l'environnement ? bah on le vire.
		tmp_env.del_env_typ (s);
		return (new VFermeture (s, t, e, tmp_env));
	} 
}
