/**
* Classe representant un If
*
* @version 0.1, 06/05/2003
* @see Expr
* @see java.lang.String
* @see Env
*/
public class If extends Expr
{
	private Expr cond;;
	private Expr e1;
	private Expr e2;

	/**
	* Constructeur de la classe If
	*
	* @param La premiere expression est la condition (de type TBool)
	* @param Les deux auters conditions doivent etre de meme types.
	*/
	public If (Expr cond, Expr e1, Expr e2)
	{
		this.cond = cond;
		this.e1 = e1;
		this.e2 = e2;
	}

	/**
	* Renvoit une String representant le If 
	*
	* return La String representant le If
	*/
	public String expr_str ()
	{
		return ("(if " + cond.expr_str() + " then " + e1.expr_str () + " else " + e2.expr_str() + ")");
	}

	/**
	* Calcule le type Type issu du If
	*
	* @param L'environnement.
	* @return Le Type renvoye par ce If
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Type typer (Env envp) throws Exception
	{
		Type te1, te2;
		/* cond _doit_ etre booleen */

		if (TBool.isTBool((cond.typer (envp))) == false) {
			throw new TypeException("If: Erreur de typage: " + cond.expr_str () + " n'est pas booleene.");
		}

		te1 = e1.typer (envp);
		te2 = e2.typer (envp);

		/* e1 et e2 doivent etre de meme type */
		if (TFonction.compareType(te1, te2))
			return (te1); /* On retourne le meme type ... */

		/* Exception */
		throw new TypeException("If: Erreur de typage: " + e1.expr_str () + " et " 
		                        + e2.expr_str () + " ne sont pas de meme type.");
	}

	/**
	* Calcule la Valeur issue de cette expression If
	*
	* @param L'environnement
	* @return La Valeur de cette Addition.
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Valeur evaluer (Env envp) throws Exception
	{
		/* Ok, on regarde si cond est bool */

		if (TBool.isTBool((cond.typer (envp))) == false) {
			throw new TypeException("If: Erreur de typage: " + cond.expr_str () + " n'est pas booleene.");
		}

		/* On regarde si e1 et e2 sont du meme type */

		if (TFonction.compareType(e1.typer (envp), e2.typer (envp))) {
			if (cond.evaluer(envp).valeur_bool())
				return (e1.evaluer(envp));
			else
				return (e2.evaluer(envp));
		} else {
			throw new TypeException("If: Erreur de typage: " + e1.expr_str () + " et " + e2.expr_str () + " ne sont pas de meme type.");
		}
	}
}
