/**
* Classe representant une modulo 
*
* @version 0.1, 06/05/2003
* @see Expr
* @see java.lang.String
* @see TEntier
* @see VEntier
* @see Env
*/
public class Modulo extends Expr
{
	private Expr e1;
	private Expr e2;

	/**
	* Constructeur de la classe Modulo
	*
	* @param Les deux expressions a droite et a gauche de la Modulo.
	*/
	public Modulo (Expr e1, Expr e2)
	{
		this.e1 = e1;
		this.e2 = e2;
	}

	/**
	* Renvoit la String representant la Modulo.
	*
	* @return Une String representant la Modulo.
	*/
	public String expr_str ()
	{
		return ("(" + e1.expr_str () + " mod " + e2.expr_str () + ")");
	}

	/**
	* Calcule le type Type issu de la Modulo.
	*
	* @param L'environnement.
	* @return Le Type de ce Modulo.
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Type typer (Env envp) throws Exception
	{
		if (TEntier.isTEntier((e1.typer (envp))) == false) {
			throw new TypeException("Modulo: Erreur de typage: " + e1.expr_str () + " n'est pas entiere.");
		}

		if (TEntier.isTEntier((e2.typer(envp))) == false) {
			throw new TypeException("Modulo: Erreur de typage: " + e2.expr_str () + " n'est pas entiere.");
		}

		return (new TEntier ());
	}

	/**
	* Calcule la Valeur issue de la Modulo.
	*
	* @param L'environnement
	* @return La Valeur de cette Modulo.
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Valeur evaluer (Env envp) throws Exception
	{
		if (TEntier.isTEntier((e1.typer (envp))) == false) {
			throw new TypeException("Modulo: Erreur de typage: " + e1.expr_str () + " n'est pas entiere.");
		}

		if (TEntier.isTEntier((e2.typer(envp))) == false) {
			throw new TypeException("Modulo: Erreur de typage: " + e2.expr_str () + " n'est pas entiere.");
		}

		return (new VEntier(e1.evaluer(envp).valeur_int() % e2.evaluer(envp).valeur_int()));
	}
}
