import java.io.*;
import java.util.*;

/**
* Classe Session, definissant les actions possibles (declaration, execution)
* lors d'une session caml.
*
* @version 0.2, 07/05/2003
* @see java.io.Serializable
* @see Declaration
* @see java.lang.String
* @see java.lang.StringBuffer
* @see TypeException
* @see ValeurException
* @see EnvException
* @see Exception
* @see Expr
* @see Env
*/
public class Session implements java.io.Serializable
{
	/* Une session est une liste d'instructions: */
	private ArrayList instructions; 
	private String filename;

	public Session ()
	{
		this.instructions = new ArrayList ();
		this.filename = null;
	}

	/**
	* Execution de la session
	*
	* @param old L'environnement de dpart.
	* @return L'environnement  la fin de l'xecution. 
	*/
	public Env executer (Env old)
	{
		Env new_envp = old.CopieEnv();
		int l, i;

		/* Execution ... */
		Object tab_ins[] = instructions.toArray();
		l = instructions.size ();

		for (i = 0; i < l; i ++) {
			if (tab_ins[i] instanceof Declaration) {
				((Declaration)tab_ins[i]).executer (new_envp);
			} else if (tab_ins[i] instanceof Expr) {
				((Expr)tab_ins[i]).executer (new_envp);
			}
		}

		return new_envp;
	}

	/**
	* Execution de la session + creation de l'environnement
	*
	* @return Le nouvel environnement, aprs cration et execution.
	*/
	public Env executer ()
	{
		Env envp = new Env ();
		return this.executer (envp);
	}

	/**
	* Ajout d'une declaration a la session. 
	*
	* @param a La declaration.
	*/
	public void ajouter_ins (Declaration a)
	{
		this.instructions.add (a);
	}

	/**
	* Ajout d'une expression a la session.
	*	
	* @param a L'expression.
	*/
	public void ajouter_ins (Expr a)
	{
		this.instructions.add (a);
	}

	/**
	* Recuperer un nom de fichier
	*
	* @return Une string contenant le nom de fichier.
	*/
	public static String get_filename ()
	{
		String l;
		BufferedReader myInput;
		myInput = new BufferedReader(new InputStreamReader(System.in));

		while (true) {
			try {
				System.out.println("Merci de donner le nom de fichier:");
				l = myInput.readLine ();
				return (l);
			} catch (IOException ioe) {
				System.out.println ("mauvaise saisie !");
			}
		}
	}

	/**
	* Sauver la session.
	*/
	public void sauve_session ()
	{
		FileOutputStream ostream;
		ObjectOutputStream p;

		if (this.filename == null)
			this.filename = get_filename();

		try {
			ostream = new FileOutputStream ("/tmp/" + this.filename);
			p = new ObjectOutputStream (ostream);

			p.writeObject (this);
			p.flush ();
			ostream.close ();
		} catch (Exception esave) {
			System.out.println ("paf casse: " + esave);
		}

		return ;
	}

	/**
	* Charger une session.
	*
	* @return La session charge.
	*/
	public static Session charge_session ()
	{
		FileInputStream istream;
		ObjectInputStream q;
		String filename = get_filename();

		Session test;
		try {
			istream = new FileInputStream ("/tmp/" + filename);
			q = new ObjectInputStream (istream);

			test = (Session)q.readObject();
			istream.close ();
			return test;
		} catch (Exception eload) {
			System.out.println ("paf casse: " + eload);
		}
		return null;
	}
}
