/**
* Classe representant une superiorite 
*
* @version 0.1, 06/05/2003
* @see Expr
* @see java.lang.String
* @see TEntier
* @see VEntier
* @see TBool
* @see VBool
* @see Env
*
*/
public class Sup extends Expr
{
	private Expr e1;
	private Expr e2;

	/**
	* Constructeur de la classe Sup
	*
	* @param Les deux expressions a droite et a gauche de Sup
	*/
	public Sup (Expr e1, Expr e2)
	{
		this.e1 = e1;
		this.e2 = e2;
	}

	/**
	* Renvoit la String representant le Sup
	*
	* @param Une String representant le Sup
	*/
	public String expr_str ()
	{
		return ("(" + e1.expr_str () + " > " + e2.expr_str () + ")");
	}

	/**
	* Calcule le type Type issu du Sup
	*
	* @param L'environnement.
	* @return Le Type de ce Sup.
	* @exception TypeException
	*            Erreur de typage.
	*/
	public Type typer (Env envp) throws Exception
	{
		if (TBool.isTBool((e1.typer (envp)))) { /* e1 bool */
			if (TBool.isTBool((e2.typer(envp))) == false) { /* e2 pas bool */
				throw new TypeException("Sup: Erreur de typage: " + e2.expr_str () + " n'est pas booleene.");
			}
			return (new TBool ());
		}

		if (TEntier.isTEntier((e1.typer (envp)))) { /* e1 int */
			if (TEntier.isTEntier((e2.typer(envp))) == false) { /* pas int */
				throw new TypeException("Sup: Erreur de typage: " + e2.expr_str () + " n'est pas entier.");
			}
			return (new TBool ());
		}

		throw new TypeException("Sup: Erreur de typage: " + e1.expr_str() + " n'est pas reconnue.");
	}

	/**
	* Calcule la Valeur issue de ce Sup.
	*
	* @param L'environnement
	* @return La Valeur de ce Sup (VBool).
	* @exception TypeException
	*            Erreur de typage
	*/
	public Valeur evaluer (Env envp) throws Exception
	{
		if (TBool.isTBool((e1.typer (envp)))) {
			if (TBool.isTBool((e2.typer(envp))) == false) {
				throw new TypeException("Sup: Erreur de typage: " + e1.expr_str () + " n'est pas booleene.");
			}

			/* seul cas qui return true: true > false */
			if (e1.evaluer(envp).valeur_bool() == true 
			 && e2.evaluer(envp).valeur_bool() == false)
				return (new VBool(true));
			else
				return (new VBool(false));
		}

		if (TEntier.isTEntier((e1.typer (envp)))) { /* e1 int */
			if (TEntier.isTEntier((e2.typer(envp))) == false) { /* pas int */
				throw new TypeException("Sup: Erreur de typage: " + e2.expr_str () + " n'est pas entier.");
			}
			return (new VBool(e1.evaluer(envp).valeur_int() > e2.evaluer(envp).valeur_int()));
		}

		throw new TypeException("Sup: Erreur de typage: " + e1.expr_str() + " n'est pas reconnue.");
	}
}
