/**
* La classe representant une fonction.
*
* @version 0.1, 06/05/2003
* @see Type
* @see java.lang.Object
* @see java.lang.String
*/

public class TFonction extends Type
{
	private Type t_gauche;
	private Type t_droite;

	/**
	* Constructeur de la classe TFonction
	*
	* @param La 'partie' gauche de la fonction
	* @param La 'partie' droite de la fonction
	*/
	public TFonction (Type gauche, Type droite)
	{
		t_gauche = gauche;
		t_droite = droite;
	}

	/**
	* Renvoit un objet String contenant la chaine relative a l'expression de la fonction.
	*
	* @return Un objet String contenant l'expression de la fonction.
	*/
	public String type_str ()
	{
		return ("(" + t_gauche.type_str() + ") -> " + t_droite.type_str ());
	}

	/**
	* Renvoit le type de la partie gauche de la fonction
	*
	* @return Un objet Type representant le type de la partie gauche de la fonction.
	*/
	public Type get_t_gauche()
	{
		return (this.t_gauche);
	}

	/**
	* Renvoit le type de la partie droite de la fonction
	*
	* @return Un objet Type representant le type de la partie droite de la fonction.
	*/
	public Type get_t_droite()
	{
		return (this.t_droite);
	}

	/**
	* Teste si les deux objets de type Type sont des instances du meme objet, 
	* et si ce sont des fonctions, que leurs parties droites et gauches sont
	* respectivement de meme types. Renvoit 'true' si c'est le cas, 
	* sinon 'false'.
	*
	* @param Les deux types a comparer.
	* @return Un booleen: true si les deux types sont identiques, false sinon.
	*/
	public static boolean compareType (Type alpha, Type beta)
	{
		if (alpha instanceof TBool && beta instanceof TBool)
			return (true);
		if (alpha instanceof TEntier && beta instanceof TEntier)
			return (true);

		if (alpha instanceof TFonction && beta instanceof TFonction)
			/* Appel recursif */
			return (TFonction.compareType(((TFonction)alpha).get_t_gauche(), ((TFonction)beta).get_t_gauche()) 
			     && TFonction.compareType(((TFonction)alpha).get_t_droite(), ((TFonction)beta).get_t_droite()));

		return (false);
	}
}
