/**
* La classe representant une fermeture.
*
* @version 0.1, 06/05/2003
* @see Valeur
* @see ValeurException
* @see java.lang.String 
* @see java.lang.Object
* @see Type
* @see Expr
* @see Env
*/
public class VFermeture extends Valeur
{
	private String s;
	private Type t;
	private Expr e;
	private Env envp;

	/**
	* Constructeur de la VFermeture.
	*
	* @param String s, le nom de cette fermeture
	* @param Type t, le type de cette fermeture
	* @param Expr e, l'expression relative a cette fermeture
	* @param Env envp, l'envirronement ratache a cette fermeture.
	*/
	public VFermeture (String s, Type t, Expr e, Env envp)
	{
		this.s = s;
		this.t = t;
		this.e = e;
		this.envp = envp;
	}

	/**
	* Renvoit une representation lisible de cette fermeture.
	*
	* @return Un objet String contenant la representation lisible de la fermeture.
	*/
	public String val_str ()
	{
		return ("< " + s + " ~> " + e.expr_str () + " , [" + envp.env_str(1) + "] >" );
	}

	/**
	* Eleve une exception: Une fonction n'a pas de valeur booleenne.
	*
	* @return Rien.
	* @exception ValeurException
	*            Une fonction n'a pas de valeur booleene.
	*/
	public boolean valeur_bool () throws ValeurException
	{
		throw new ValeurException ("Une fonction n'a pas de valeur booleene.\n");
	}

	/**
	* Eleve une exception: Une fonction n'a pas de valeur entiere.
	*
	* @return Rien.
	* @exception ValeurException
	*            Une fonction n'a pas de valeur entiere.
	*/
	public int valeur_int () throws ValeurException
	{
		throw new ValeurException ("Une fonction n'a pas de valeur entiere.\n");
	}

	/**
	* Renvoit l'objet String du nom de la VFermeture.
	*
	* @return L'objet String contenant le nom de la VFermeture.
	*/
	public String getString () { return (s); }

	/**
	* Renvoit l'objet Type du type de la VFermeture.
	*
	* @return L'objet Type contenant le type de la VFermeture.
	*/
	public Type getType () { return (t); }

	/**
	* Renvoit l'objet Expr de l'expression de la VFermeture.
	*
	* @return L'objet Expr contenant l'expression de la VFermeture.
	*/
	public Expr getExpr () { return (e); }

	/**
	* Renvoit l'objet Env de l'envirronement de la VFermeture.
	*
	* @return L'objet Env contenant l'envirronement de la VFermeture.
	*/
	public Env getEnv () { return (envp); }
}
