/* Represente l'environnement: Liste de Variable * Type * Valeur
 * Utilisation des HashTables.
 */

import java.util.*;

/**
* La classe representant un envirronement: Une hashtable de String
* pointant sur des ObjValTyp
*
* @version 0.1, 06/05/2003 
* @see java.lang.String
* @see java.lang.StringBuffer
* @see ObjValTyp
* @see java.io.Serializable
* @see java.util.Hashtable
* @see java.lang.Object
*/
public class Env implements java.io.Serializable
{
	private Hashtable envp;

	/**
	* Constructeur de la classe Env.
	*/
	public Env () 
	{
		envp = new Hashtable ();
	}

	/**
	* Constructeur de la classe Env, prenant une HashTable
	* (principalement utilise pour faire une copie.)
	*
	* @param u Un objet Hashtable, representant un environnement.
	*/
	public Env (Hashtable u)
	{
		envp = (Hashtable)u.clone ();
	}

	/**
	* Renvoit une chaine de caracteres contenant une String representant
	* l'environnement. Ceci est possible de 2 facons differentes
	* (mode == 1: pour la VFermeture ou mode == 2: pour l'affichage humain)
	*
	* @param mode Le mode de sortie.
	* @return Une chaine de caracteres.
	*/
	public String env_str (int mode)
	{
		StringBuffer sb = new StringBuffer ("");
		String tmp;
		ObjValTyp tmpobj;
		int i = 0;

		for (Enumeration e = envp.keys ();
		     e.hasMoreElements();) {

			tmp = (String) e.nextElement();
			tmpobj = (ObjValTyp)envp.get(tmp);

			if (mode == 1) {
				if (i != 0) sb.append (";");
				sb.append ("(" + tmp + "," + tmpobj.getValeur() + ")");
			} else if (mode == 2) {
				if (i != 0) sb.append (";");
				sb.append ("(" + tmp + "=" + tmpobj.getValeur() + ")");
			} else if (mode == 3) {
				if (i != 0) sb.append (" and ");
				sb.append ("(" + tmp + "=" + tmpobj.getValeur() + ")");
			}
			i ++;
		}

		return (sb.toString());
	}

	/**
	* Renvoit une valeur rattachee a une variable du nom contenu dans une String.
	* Si la variable n'existe pas, on envoit une exception de type EnvException
	*
	* @param s L'objet String contenant le nom de la variable que l'on veut.
	* @return L'objet Valeur, contenant le type de la variable.
	* @exception EnvException
	*            Si la variable n'existe pas.
	*
	*/
	public Valeur val_de (String s) throws EnvException
	{
		ObjValTyp tmp;
		tmp = (ObjValTyp)envp.get(s);
		if (tmp == null) {
			throw new EnvException ("Variable \"" + s + "\" non definie (val_de).");
		}
		return (tmp.getValeur ());
	}

	/**
	* Renvoit un type rattache a une variable du nom contenu dans une String.
	* Si la variable n'existe pas, on envoit une exception de type EnvException.
	*
	* @param s L'objet String, contenant le nom de la variable que l'on veut.
	* @return L'objet Type, contenant le type de la variable.
	* @exception EnvException
	*            Si la variable n'existe pas.
	*/
	public Type typ_de (String s) throws EnvException
	{
		ObjValTyp tmp;
		tmp = (ObjValTyp)envp.get(s);
		if (tmp == null) {
			throw new EnvException ("Variable \"" + s + "\" non definie (typ_de).");
		}
		return (tmp.getType ());
	}

	/**
	* Ajoute une Valeur rattachee a un nom de variable dans l'envirronement.
	*
	* @param s le nom de la variable
	* @param v l'object Valeur a rattacher.
	*/
	public void ajt_env_val (String s, Valeur v)
	{
		envp.put(s, new ObjValTyp(v));
	}

	/**
	* Ajoute un Type ratache a un nom de variable dans l'envirronement.
	*
	* @param s le nom de la variable
	* @param v l'objet Valeur a rattacher.
	*/
	public void ajt_env_typ (String s, Type t)
	{
		envp.put(s, new ObjValTyp(t));
	}

	/**
	* Ajout un couple Valeur/Type rattache a un nom de variable 
	* dans l'envirronement.
	*
	* @param s le nom de la variable
	* @param v l'objet Valeur a rattacher.
	* @param t l'objet Type de cette variable.
	*/
	public void ajt_env (String s, Type t, Valeur v)
	{
		envp.put(s, new ObjValTyp(v, t));
	}

	/**
	* Efface une variable dans l'envirronement.
	*
	* @param s objet String contenant le nom de cette variable.
	*/
	public void del_env_val (String s)
	{
		ObjValTyp tmp;
		tmp = (ObjValTyp)envp.get(s);
		if (tmp != null) {
			envp.remove(s);
		}
	}

	/**
	* Efface une variable dans l'envirronement.
	*
	* @param s objet String contenant le nom de cette variable.
	*/
	public void del_env_typ (String s)
	{
		ObjValTyp tmp;
		tmp = (ObjValTyp)envp.get(s);
		if (tmp != null) {
			envp.remove(s);
		}
	}

	/**
	* Retourner la hash-table associe  un environnement
	*
	* @return La hashtable
	*/
	public Hashtable getTable ()
	{
		return (this.envp);
	}

	/**
	* Copie d'un environement.
	*
	* @param old un objet Env existant, a copier
	* @return Un nouvel environnement, similaire a celui passe en argument.
	*/
	public static Env CopieEnv (Env old)
	{
		return (new Env(old.envp));
	}

	/**
	* Copie d'un environnement.
	*
	* @return La copie de l'environnement courant.
	*/
	public Env CopieEnv ()
	{
		Env tmp = new Env (this.envp);
		return (tmp);
	}

	public Env MergeWithEnv (Env mini_env)
	{
		Env my_env = this.CopieEnv();
		String tmpstr;
		ObjValTyp tmpobj;

		for (Enumeration e = mini_env.getTable().keys ();
			e.hasMoreElements();) {

			/* on recupere le nom d'une variable du petit env. */
			tmpstr = (String) e.nextElement();

			/* on recupere son objet associ */
			tmpobj = (ObjValTyp)mini_env.getTable().get(tmpstr);

			/* On colle cela dans le my_env, apres avoir evaluation. */

			my_env.ajt_env (tmpstr, tmpobj.getType(), 
			                        tmpobj.getValeur());
		}
		return my_env;
	}
}
